""" Code is generated by ucloud-model, DO NOT EDIT IT. """

import typing


from ucloud.core.client import Client
from ucloud.services.cube.schemas import apis


class CubeClient(Client):
    def __init__(
        self, config: dict, transport=None, middleware=None, logger=None
    ):
        super(CubeClient, self).__init__(config, transport, middleware, logger)

    def create_cube_pod(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """CreateCubePod - 创建Pod

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Pod** (str) - (Required) base64编码的Pod的yaml
        - **SubnetId** (str) - (Required) 子网Id
        - **VPCId** (str) - (Required) VPCId
        - **Zone** (str) - (Required) 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **ChargeType** (str) - 计费模式。枚举值为： \\ > Year，按年付费； \\ > Month，按月付费；\\ > Postpay， \\ 后付费；默认为后付费
        - **CpuPlatform** (str) - Cpu平台（V6、A2），默认V6
        - **Group** (str) - pod所在组
        - **Name** (str) - pod的名字
        - **Quantity** (int) - 购买时长。默认:值 1。 月付时，此参数传0，代表购买至月末。
        - **Tag** (str) - 业务组。默认：Default（Default即为未分组）

        **Response**

        - **Action** (str) - 操作名称
        - **CubeId** (str) - cube的资源Id
        - **Pod** (str) - base64编码的yaml
        - **RetCode** (int) - 返回码

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.CreateCubePodRequestSchema().dumps(d)

        # build options
        kwargs["max_retries"] = 0  # ignore retry when api is not idempotent

        resp = self.invoke("CreateCubePod", d, **kwargs)
        return apis.CreateCubePodResponseSchema().loads(resp)

    def delete_cube_pod(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """DeleteCubePod - 删除Pod

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **CubeId** (str) - cubeid和uid任意一个（必须）
        - **ReleaseEIP** (bool) - 删除cube时是否释放绑定的EIP。默认为false。
        - **Uid** (str) - cubeid和uid任意一个（必须）
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**


        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.DeleteCubePodRequestSchema().dumps(d)

        resp = self.invoke("DeleteCubePod", d, **kwargs)
        return apis.DeleteCubePodResponseSchema().loads(resp)

    def get_cube_extend_info(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """GetCubeExtendInfo - 获取Cube的额外信息

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **CubeIds** (str) - (Required) id列表以逗号(,)分割
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **ExtendInfo** (list) - 见 **CubeExtendInfo** 模型定义

        **Response Model**

        **EIPAddr**

        - **IP** (str) - IP地址
        - **OperatorName** (str) - 线路名称BGP或者internalation

        **EIPSet**

        - **Bandwidth** (int) - EIP带宽值
        - **BandwidthType** (int) - 带宽类型0标准普通带宽，1表示共享带宽
        - **CreateTime** (int) - EIP创建时间
        - **EIPAddr** (list) - 见 **EIPAddr** 模型定义
        - **EIPId** (str) - EIPId
        - **PayMode** (str) - 付费模式，带宽付费或者流量付费
        - **Resource** (str) - EIP绑定对象的资源Id
        - **Status** (str) - EIP状态，表示使用中或者空闲
        - **Weight** (int) - EIP权重

        **CubeExtendInfo**

        - **CubeId** (str) - Cube的Id
        - **Eip** (list) - 见 **EIPSet** 模型定义
        - **Expiration** (int) - 资源有效期
        - **Name** (str) - Cube的名称
        - **Tag** (str) - 业务组名称

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.GetCubeExtendInfoRequestSchema().dumps(d)

        resp = self.invoke("GetCubeExtendInfo", d, **kwargs)
        return apis.GetCubeExtendInfoResponseSchema().loads(resp)

    def get_cube_pod(self, req: typing.Optional[dict] = None, **kwargs) -> dict:
        """GetCubePod - 获取Pod的详细信息

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **CubeId** (str) - CubeId和Uid任意一个
        - **Uid** (str) - CubeId和Uid任意一个
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **Pod** (str) - base64编码的pod的yaml

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.GetCubePodRequestSchema().dumps(d)

        resp = self.invoke("GetCubePod", d, **kwargs)
        return apis.GetCubePodResponseSchema().loads(resp)

    def list_cube_pod(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """ListCubePod - 获取Pods列表

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **Group** (str) - 组名称
        - **Limit** (int) - 默认20
        - **Offset** (int) - 默认0
        - **SubnetId** (str) - 子网Id
        - **VPCId** (str) - VPC的Id
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **Pods** (list) - Pod列表，每条数据都做了base64编码
        - **TotalCount** (int) - Cube的总数

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.ListCubePodRequestSchema().dumps(d)

        resp = self.invoke("ListCubePod", d, **kwargs)
        return apis.ListCubePodResponseSchema().loads(resp)

    def modify_cube_extend_info(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """ModifyCubeExtendInfo - 修改Cube额外信息

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **CubeId** (str) - (Required) cube的id
        - **Name** (str) - 修改的名字，规则（^[a-zA-Z0-9-_.\u4e00-\u9fa5]{1,32}）
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**


        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.ModifyCubeExtendInfoRequestSchema().dumps(d)

        resp = self.invoke("ModifyCubeExtendInfo", d, **kwargs)
        return apis.ModifyCubeExtendInfoResponseSchema().loads(resp)

    def renew_cube_pod(
        self, req: typing.Optional[dict] = None, **kwargs
    ) -> dict:
        """RenewCubePod - 更新Pod

        **Request**

        - **ProjectId** (str) - (Config) 项目ID。不填写为默认项目，子帐号必须填写。 请参考 `GetProjectList接口 <https://docs.ucloud.cn/api/summary/get_project_list.html>`_
        - **Region** (str) - (Config) 地域。 参见  `地域和可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_
        - **CubeId** (str) - (Required) 容器Id
        - **Pod** (str) - (Required) base64编码的Pod的yaml
        - **Zone** (str) - 可用区。参见  `可用区列表 <https://docs.ucloud.cn/api/summary/regionlist.html>`_

        **Response**

        - **Pod** (str) - base64编码过的yaml，需要解码获取信息

        """
        # build request
        d = {
            "ProjectId": self.config.project_id,
            "Region": self.config.region,
        }
        req and d.update(req)
        d = apis.RenewCubePodRequestSchema().dumps(d)

        resp = self.invoke("RenewCubePod", d, **kwargs)
        return apis.RenewCubePodResponseSchema().loads(resp)



import time
import typing
import logging
from ucloud.core import exc


MAX_BACKOFF_INTERVAL = 10

logger = logging.getLogger(__name__)


class WaitTimeoutException(exc.UCloudException):
    pass


class StateConf:
    """StateConf is the utilities class to wait the state return by refresh function achieve the specific state,
    the generally usage is wait the cloud resource, such as uhost, udb ... is
    ready after created.
    """

    def __init__(
        self,
        pending: typing.List[str],
        target: typing.List[str],
        refresh: typing.Callable,
        timeout: float,
        startup_delay: float = 0,
        min_backoff_interval: float = 0.1,
        max_backoff_interval: float = MAX_BACKOFF_INTERVAL,
    ):
        self.pending = pending
        self.target = target
        self.refresh = refresh
        self.timeout = timeout
        self.startup_delay = startup_delay
        self.min_backoff_interval = min_backoff_interval
        self.max_backoff_interval = max_backoff_interval

    def wait(self):
        start_time = time.time()
        interval = self.min_backoff_interval

        # delay on start up
        time.sleep(self.startup_delay)

        # waiting for state changed to target state
        while time.time() - start_time < self.timeout:
            state = self.refresh()

            if state in self.pending:
                time.sleep(interval)
                interval *= 2
                if interval > self.max_backoff_interval:
                    interval = self.max_backoff_interval
                logger.info(
                    "waiting state for {self.refresh}, got state {state}".format(
                        self=self, state=state
                    )
                )
                continue

            if state in self.target:
                return

        raise WaitTimeoutException(
            "wait timeout {self.timeout}s for {self.refresh}".format(self=self)
        )


def wait_for_state(
    pending: typing.List[str],
    target: typing.List[str],
    refresh: typing.Callable,
    timeout: float,
    startup_delay: float = 0,
    min_backoff_interval: float = 0.1,
    max_backoff_interval: float = MAX_BACKOFF_INTERVAL,
):
    """wait_for_state is a utilities function to wait the state return by refresh function achieve the specific state,
    the generally usage is wait the cloud resource, such as uhost, udb ... is
    ready after created.

    >>> wait_for_state(
    ...     pending=["pending"],
    ...     target=["running"],
    ...     refresh=lambda: "running",
    ...     timeout=0.5,
    ... )

    :param pending: pending is the list of pending state, the state is returned by refresh function
    :param target: target is the list of target state, it is usually the terminate state, eg. running and fail
    :param refresh: the customized refresh function, expect no arguments and return a state
    :param timeout: timeout is the total time to wait state is achieved
    :param startup_delay: the time to wait before first refresh function is called
    :param min_backoff_interval: the backoff time for first refresh interval
    :param max_backoff_interval: the max backoff time for refresh interval
    """
    conf = StateConf(
        pending=pending,
        target=target,
        refresh=refresh,
        timeout=timeout,
        startup_delay=startup_delay,
        min_backoff_interval=min_backoff_interval,
        max_backoff_interval=max_backoff_interval,
    )
    return conf.wait()



""" Code is generated by ucloud-model, DO NOT EDIT IT. """

from ucloud.core.typesystem import schema, fields


class UMemBackupSetSchema(schema.ResponseSchema):
    """UMemBackupSet - DescribeUMemBackup"""

    fields = {
        "BackupId": fields.Str(required=True, load_from="BackupId"),
        "BackupName": fields.Str(required=True, load_from="BackupName"),
        "BackupType": fields.Str(required=True, load_from="BackupType"),
        "BlockCount": fields.Int(required=True, load_from="BlockCount"),
        "CreateTime": fields.Int(required=True, load_from="CreateTime"),
        "State": fields.Str(required=True, load_from="State"),
    }


class UMemBlockInfoSchema(schema.ResponseSchema):
    """UMemBlockInfo - 分布式redis 分片信息"""

    fields = {
        "BlockId": fields.Str(required=True, load_from="BlockId"),
        "BlockPort": fields.Int(required=True, load_from="BlockPort"),
        "BlockSize": fields.Int(required=True, load_from="BlockSize"),
        "BlockSlotBegin": fields.Int(required=True, load_from="BlockSlotBegin"),
        "BlockSlotEnd": fields.Int(required=True, load_from="BlockSlotEnd"),
        "BlockState": fields.Str(required=True, load_from="BlockState"),
        "BlockUsedSize": fields.Int(required=True, load_from="BlockUsedSize"),
        "BlockVip": fields.Str(required=True, load_from="BlockVip"),
    }


class UMemPriceSetSchema(schema.ResponseSchema):
    """UMemPriceSet - DescribeUMemPrice"""

    fields = {
        "ChargeType": fields.Str(required=False, load_from="ChargeType"),
        "ListPrice": fields.Int(required=False, load_from="ListPrice"),
        "OriginalPrice": fields.Int(required=False, load_from="OriginalPrice"),
        "Price": fields.Int(required=False, load_from="Price"),
    }


class UMemSpaceAddressSetSchema(schema.ResponseSchema):
    """UMemSpaceAddressSet - DescribeUMemSpace"""

    fields = {
        "IP": fields.Str(required=False, load_from="IP"),
        "Port": fields.Int(required=False, load_from="Port"),
    }


class UMemSpaceSetSchema(schema.ResponseSchema):
    """UMemSpaceSet - DescribeUMemSpace"""

    fields = {
        "Address": fields.List(UMemSpaceAddressSetSchema()),
        "ChargeType": fields.Str(required=False, load_from="ChargeType"),
        "CreateTime": fields.Int(required=False, load_from="CreateTime"),
        "ExpireTime": fields.Int(required=False, load_from="ExpireTime"),
        "Name": fields.Str(required=False, load_from="Name"),
        "Protocol": fields.Str(required=False, load_from="Protocol"),
        "RewriteTime": fields.Int(required=True, load_from="RewriteTime"),
        "Size": fields.Int(required=False, load_from="Size"),
        "SpaceId": fields.Str(required=False, load_from="SpaceId"),
        "State": fields.Str(required=False, load_from="State"),
        "SubnetId": fields.Str(required=False, load_from="SubnetId"),
        "Tag": fields.Str(required=True, load_from="Tag"),
        "Type": fields.Str(required=False, load_from="Type"),
        "UsedSize": fields.Int(required=False, load_from="UsedSize"),
        "VPCId": fields.Str(required=False, load_from="VPCId"),
        "Zone": fields.Str(required=False, load_from="Zone"),
    }


class PriceDataSetSchema(schema.ResponseSchema):
    """PriceDataSet - 升降级价格"""

    fields = {
        "CustomPrice": fields.Int(required=False, load_from="CustomPrice"),
        "PurchaseValue": fields.Int(required=False, load_from="PurchaseValue"),
        "TotalPrice": fields.Int(required=False, load_from="TotalPrice"),
    }


class UMemcacheGroupSetSchema(schema.ResponseSchema):
    """UMemcacheGroupSet - DescribeUMemcacheGroup"""

    fields = {
        "ChargeType": fields.Str(required=False, load_from="ChargeType"),
        "ConfigId": fields.Str(required=False, load_from="ConfigId"),
        "CreateTime": fields.Int(required=False, load_from="CreateTime"),
        "ExpireTime": fields.Int(required=False, load_from="ExpireTime"),
        "GroupId": fields.Str(required=False, load_from="GroupId"),
        "ModifyTime": fields.Int(required=False, load_from="ModifyTime"),
        "Name": fields.Str(required=False, load_from="Name"),
        "Port": fields.Int(required=False, load_from="Port"),
        "Size": fields.Int(required=False, load_from="Size"),
        "State": fields.Str(required=False, load_from="State"),
        "SubnetId": fields.Str(required=False, load_from="SubnetId"),
        "Tag": fields.Str(required=False, load_from="Tag"),
        "UsedSize": fields.Int(required=False, load_from="UsedSize"),
        "VPCId": fields.Str(required=False, load_from="VPCId"),
        "Version": fields.Str(required=False, load_from="Version"),
        "VirtualIP": fields.Str(required=False, load_from="VirtualIP"),
    }


class UMemcachePriceSetSchema(schema.ResponseSchema):
    """UMemcachePriceSet - DescribeUMemcachePrice"""

    fields = {
        "ChargeType": fields.Str(required=False, load_from="ChargeType"),
        "ListPrice": fields.Int(required=False, load_from="ListPrice"),
        "OriginalPrice": fields.Int(required=False, load_from="OriginalPrice"),
        "Price": fields.Int(required=False, load_from="Price"),
    }


class URedisBackupSetSchema(schema.ResponseSchema):
    """URedisBackupSet - DescribeURedisBackup"""

    fields = {
        "BackupId": fields.Str(required=False, load_from="BackupId"),
        "BackupName": fields.Str(required=False, load_from="BackupName"),
        "BackupSize": fields.Int(required=False, load_from="BackupSize"),
        "BackupTime": fields.Int(required=False, load_from="BackupTime"),
        "BackupType": fields.Str(required=False, load_from="BackupType"),
        "GroupId": fields.Str(required=False, load_from="GroupId"),
        "GroupName": fields.Str(required=False, load_from="GroupName"),
        "State": fields.Str(required=False, load_from="State"),
        "Zone": fields.Str(required=False, load_from="Zone"),
    }


class URedisConfigSetSchema(schema.ResponseSchema):
    """URedisConfigSet - 主备Redis配置文件信息"""

    fields = {
        "ConfigId": fields.Str(required=False, load_from="ConfigId"),
        "CreateTime": fields.Int(required=False, load_from="CreateTime"),
        "Description": fields.Str(required=False, load_from="Description"),
        "IsModify": fields.Str(required=False, load_from="IsModify"),
        "ModifyTime": fields.Int(required=False, load_from="ModifyTime"),
        "Name": fields.Str(required=False, load_from="Name"),
        "RegionFlag": fields.Bool(required=False, load_from="RegionFlag"),
        "State": fields.Str(required=False, load_from="State"),
        "Version": fields.Str(required=False, load_from="Version"),
        "Zone": fields.Str(required=False, load_from="Zone"),
    }


class URedisGroupSetSchema(schema.ResponseSchema):
    """URedisGroupSet - DescribeURedisGroup"""

    fields = {
        "AutoBackup": fields.Str(required=False, load_from="AutoBackup"),
        "BackupTime": fields.Int(required=False, load_from="BackupTime"),
        "ChargeType": fields.Str(required=False, load_from="ChargeType"),
        "ConfigId": fields.Str(required=False, load_from="ConfigId"),
        "CreateTime": fields.Int(required=False, load_from="CreateTime"),
        "ExpireTime": fields.Int(required=False, load_from="ExpireTime"),
        "GroupId": fields.Str(required=False, load_from="GroupId"),
        "GroupName": fields.Str(required=False, load_from="GroupName"),
        "HighAvailability": fields.Str(
            required=False, load_from="HighAvailability"
        ),
        "MemorySize": fields.Int(required=False, load_from="MemorySize"),
        "ModifyTime": fields.Int(required=False, load_from="ModifyTime"),
        "Name": fields.Str(required=False, load_from="Name"),
        "Port": fields.Int(required=False, load_from="Port"),
        "Protocol": fields.Str(required=False, load_from="Protocol"),
        "RewriteTime": fields.Int(required=True, load_from="RewriteTime"),
        "Role": fields.Str(required=True, load_from="Role"),
        "Size": fields.Int(required=False, load_from="Size"),
        "SlaveZone": fields.Str(required=False, load_from="SlaveZone"),
        "State": fields.Str(required=False, load_from="State"),
        "SubnetId": fields.Str(required=False, load_from="SubnetId"),
        "Tag": fields.Str(required=False, load_from="Tag"),
        "Type": fields.Str(required=False, load_from="Type"),
        "UsedSize": fields.Int(required=False, load_from="UsedSize"),
        "VPCId": fields.Str(required=True, load_from="VPCId"),
        "Version": fields.Str(required=False, load_from="Version"),
        "VirtualIP": fields.Str(required=False, load_from="VirtualIP"),
        "Zone": fields.Str(required=False, load_from="Zone"),
    }


class URedisPriceSetSchema(schema.ResponseSchema):
    """URedisPriceSet - 主备Redis价格"""

    fields = {
        "ChargeType": fields.Str(required=False, load_from="ChargeType"),
        "ListPrice": fields.Int(required=False, load_from="ListPrice"),
        "OriginalPrice": fields.Int(required=True, load_from="OriginalPrice"),
        "Price": fields.Int(required=False, load_from="Price"),
    }


class URedisSlowlogSetSchema(schema.ResponseSchema):
    """URedisSlowlogSet - DescribeURedisSlowlog"""

    fields = {
        "Command": fields.Str(required=False, load_from="Command"),
        "SpendTime": fields.Int(required=False, load_from="SpendTime"),
        "StartTime": fields.Int(required=False, load_from="StartTime"),
    }


class URedisVersionSetSchema(schema.ResponseSchema):
    """URedisVersionSet - DescribeURedisVersion"""

    fields = {
        "Version": fields.Str(required=False, load_from="Version"),
    }



""" Code is generated by ucloud-model, DO NOT EDIT IT. """

from ucloud.core.typesystem import schema, fields


class UFSVolumeInfo2Schema(schema.ResponseSchema):
    """UFSVolumeInfo2 - 文件系统信息"""

    fields = {
        "CreateTime": fields.Int(required=False, load_from="CreateTime"),
        "ExpiredTime": fields.Int(required=False, load_from="ExpiredTime"),
        "IsExpired": fields.Str(required=False, load_from="IsExpired"),
        "MaxMountPointNum": fields.Int(
            required=True, load_from="MaxMountPointNum"
        ),
        "ProtocolType": fields.Str(required=True, load_from="ProtocolType"),
        "Remark": fields.Str(required=False, load_from="Remark"),
        "Size": fields.Int(required=False, load_from="Size"),
        "StorageType": fields.Str(required=True, load_from="StorageType"),
        "Tag": fields.Str(required=False, load_from="Tag"),
        "TotalMountPointNum": fields.Int(
            required=True, load_from="TotalMountPointNum"
        ),
        "UsedSize": fields.Int(required=False, load_from="UsedSize"),
        "VolumeId": fields.Str(required=True, load_from="VolumeId"),
        "VolumeName": fields.Str(required=True, load_from="VolumeName"),
    }



import typing

from ucloud.core import exc


class Field(object):
    def __init__(
        self,
        required: bool = False,
        default: typing.Any = None,
        dump_to: str = None,
        load_from: str = None,
        strict: bool = None,
        **kwargs
    ):
        self.required = required
        self.default = default
        self.dump_to = dump_to
        self.load_from = load_from
        self.options = kwargs
        self.strict = bool(strict)  # None as False

    def dumps(self, value, **kwargs):
        raise NotImplementedError

    def loads(self, value, **kwargs):
        raise NotImplementedError

    @staticmethod
    def fail(name, expected, got):
        msg = "invalid field {}, expect {}, got {}".format(name, expected, got)
        raise exc.ValidationException(msg)


class Schema(object):
    fields = {}

    def __init__(
        self,
        required: bool = False,
        default: typing.Union[typing.Callable, typing.Any] = dict,
        dump_to: str = None,
        load_from: str = None,
        strict: bool = False,
        case_sensitive: bool = False,
        **kwargs
    ):
        self.required = required
        self.default = default
        self.dump_to = dump_to
        self.load_from = load_from
        self.options = kwargs
        self.strict = strict
        self.case_sensitive = case_sensitive

    def dumps(self, d: dict) -> dict:
        raise NotImplementedError

    def loads(self, d: dict) -> dict:
        raise NotImplementedError
